package lsjsample;

import java.io.PrintWriter;
import java.sql.Connection;
import java.text.SimpleDateFormat;
import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.littlesoft.LSApplicationException;
import jp.littlesoft.data.Column;
import jp.littlesoft.data.Row;
import jp.littlesoft.data.event.TableEvent;
import jp.littlesoft.sql.SQL;
import jp.littlesoft.web.ICheckableRequest;
import jp.littlesoft.web.SQLActionPage;
import jp.littlesoft.web.WebSQLTable;

/**
 * Little J-Programing Sample<br>
 * 社員一覧の参照画面<br><br>
 * 
 * ---------------------------------------------------------------------------------------<br><br>
 * 
 * Ajax呼び出しへのレスポンス例です。<br>
 * this.setUseAjax(true);  // 必須です。<br>
 * 
 * Ajax呼び出しの場合、showは使用せず、<br>
 * this.writeAjaxResponse(String)<br>
 * でHTMLの断片を書き出します。<br><br>
 * 
 * HTMLの断片をテンプレートで編集したい場合は<br>
 * getMergedString([テンプレートファイル名])<br>
 * で、ActionPageの中身とマージします。<br><br>
 * 
 * 書き出した内容は、JSONオブジェクトとしてクライアントJavaScriptのonComplete(response)に返されます。<br>
 * response.contentがthis.writeAjaxResponse()で書き出した文字列です。<br><br>
 * 
 * responseには上記の他、<br>
 * ActionPageのパブリックフィールド、ActionPage内のWebDataTableのプロパティなどがセットされています。<br>
 * ※詳しくはwriteAjaxResponse()のソースを見てください。<br><br>
 * 
 * もっと複雑な内容をresponseに入れたい場合は、<br>
 * writeAjaxResponse(String content, org.json.JSONObject responseObject)<br>
 * を使用してくだい。<br>
 * 予めJSONObjectを作成して、必要なオブジェクトをputしてからwriteAjaxResponseを呼び出すことで<br>
 * どのような内容も、クライアントJavaScriptに返却できるようになります。<br>
 * <br>
 * ---------------------------------------------------------------------------------------<br><br>
 * 
 * ページの分割取得の例です。<br/>
 * tableEmp.setPageDivReserve(true);  // 必須です。<br><br>
 * 
 * ページの分割取得を使用している場合は、ページ移動メソッド「pageFirst／pagePrevious／pageNext／pageLast」の<br>
 * 引数にデータベースコネクションが必要です。<br>
 * Connection conn = getConnection();<br>
 * tableEmp.pageFirst(conn);<br>
 * conn.close();<br>
 * 
 * <br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class EmpPage extends SQLActionPage {

  /**
   * public フィールドはVelocityテンプレートからアクセス可能なフィールドとなります。
   */
  
  /**
   * ログイン情報(現在時刻)
   */
  public String now = null;
  
  /**
   * 画面出力メッセージ
   */
  public String messages = "";

  /**
   * public のWebSQLTableは WebSQLTable.PageContextというオブジェクトに変換され
   * Velocityテンプレートからアクセス可能となります。
   */
  
  /**
   * WebSQLTableオブジェクト(社員情報テーブル)
   * SELECT * FROM T_EMP WHERE T_EMP.DEPT_CODE = :deptCode
   * ORDER BY T_EMP.NAME
   */
  public WebSQLTable<DB.Employee> tableEmp = new WebSQLTable<DB.Employee>() {{
    select()
      .where($$.deptCode).is(":deptCode").end
      .orderBy($$.name)
    .prepare();
    // 社員IDはDB登録直前に取り付けるので、NULL可に設定しておく
    $column($$.id).setNullable(true);
  }};
  
  /**
   * WebSQLTableオブジェクト(所属マスタテーブル)
   * SELECT * FROM M_DEPT ORDER BY M_DEPT.DEPT_NAME
   */
  public WebSQLTable<DB.Department> tableDept = new WebSQLTable<DB.Department>(){{
    select().orderBy($$.name).prepare();
  }};

  /**
   * WebSQLTableオブジェクト(社員情報採番テーブル)
   * SELECT * FROM T_EMP_SEQ
   */
  public WebSQLTable<DB.EmployeeSeq> tableEmpSeq = new WebSQLTable<DB.EmployeeSeq>(){{
    select().prepare();
  }};

  /**
   * UpdateSQLオブジェクト(社員情報採番テーブル)<br/>
   * UPDATE T_EMP_SEQ SET SEQ = SEQ + 1
   */
  private SQL<DB.EmployeeSeq> updateEmpSeq = new SQL<DB.EmployeeSeq>(){{
    update().set(
      $($$.value).accept($($$.value).add(1))
    ).prepare();
  }};
  
  /**
   * Velocityテンプレートファイル(社員一覧画面)
   */
  private static String empTemplate = "EmpPage.html";

  /**
   * Velocityテンプレートファイル(社員一覧画面(リスト))
   */
  private static String empTableTemplate = "EmpPageTable.html";

  /**
   * コンストラクタ。
   * 
   * @throws Exception 例外発生時
   */
  public EmpPage() {
  }
  
  /**
   * コンストラクタ。
   * 
   * @param request リクエスト
   * @param response レスポンス
   * @throws Exception 例外発生時
   */
  public EmpPage(HttpServletRequest request, HttpServletResponse response) throws Exception {

    super(request, response);
    
    setUseAjax(true);             // Ajax呼び出しを使用します。
    setTemplateName(empTemplate); // HTMLテンプレートを指定します。
    setStrictSession(true);       // バックボタンによる不正なリクエストをキャンセルし、最終ページを強制出力する指定です。
    /*
     * 社員情報テーブルオブジェクト(tableEmp)の設定
     */
    tableEmp.setContainRowNumInErrors(true);
    tableEmp.addTableListener(this);                // テーブルイベントを監視します。
    tableEmp.setShowRows(5);                        // Velocityテンプレートに一覧表示する際の行数を指定します。
    //tableEmp.setPageDivReserve(false);              // ページの分割取得を使用するか否かを設定します。(ORACLE、PostgreSQL及びMySQLのみサポート)
    Connection conn = getConnection();
    try {
      // 選択リストを作成する為、所属 テーブルを読み込んでおきます。
      tableDept.activate(conn);
      // 空の社員テーブルを読み込み
      tableEmp.getParams().setValue("deptCode", null);
      tableEmp.activate(conn);
    } finally {
      // コネクションを閉じる
      conn.close();
    }
  }

  /**
   * 各アクションメソッド開始前に呼ばれるメソッドです。
   * 
   * @param methodName メソッド名
   */
  @Override
  protected void onStartAction(String methodName) {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy'年'MM'月'dd'日'(E) HH:mm:ss");
    now = sdf.format(new java.util.Date());
  }
  
  @Override
  protected void onDefaultCheckRequestError(String methodName, ArrayList<ICheckableRequest> errorTableList, boolean showAlert) {
    //メッセージ初期化
    messages = "";
    super.onDefaultCheckRequestError(methodName, errorTableList, showAlert);
  }
  /**
   * 社員テーブルをDBから読込、テーブルオブジェクトに格納します。
   * 
   * @throws Exception 例外発生時
   */
  @DefaultCheckReqest(isEnabled=false)
  public void searchEmpList() throws Exception {
    //メッセージ初期化
    messages = "";
    //コネクション取得
    Connection conn = getConnection();
    try {
      tableEmp.checkRequest(getRequest());
      //データベースを検索し、結果をWebSQLTableオブジェクトに格納する
      tableEmp.activate(conn);
      //Ajaxレスポンスを返却(テンプレートと変数をマージした内容)
      writeAjaxResponse(getMergedString(empTableTemplate));
    } finally {
      //コネクションを閉じる
      conn.close(); 
    }
  }
  
  /**
   * 指定されたカラムで社員テーブル行の並び替えをします。
   * 
   * @param colName カラム名
   * @throws Exception 例外発生時
   */
  public void sort(String colName) throws Exception  {
    
    
    //先頭ページへ移動
    tableEmp.pageFirst();
    //引数で指定されたカラム名でソート
    tableEmp.sortRows(colName);
    //Ajaxレスポンスを返却(テンプレートと変数をマージした内容)
    writeAjaxResponse(getMergedString(empTableTemplate));
  }
  
  /**
   * 一覧に表示する内容をスクロールします。
   * 
   * @param direction ページング方向
   * @throws Exception 例外発生時
   */
  public void paging(String direction) throws Exception {
    // 先頭ページへ移動
    if ("first".equals(direction)) {
      tableEmp.pageFirst();
      // 前ページへ移動
    } else if ("prev".equals(direction)) {
      tableEmp.pagePrevious();
      // 次ページへ移動
    } else if ("next".equals(direction)) {
      tableEmp.pageNext();
      // 最終ページへ移動
    } else if ("last".equals(direction)) {
      tableEmp.pageLast();
    }
    // Ajaxレスポンスを返却(テンプレートと変数をマージした内容)
    writeAjaxResponse(getMergedString(empTableTemplate));
  }
  
  /**
   * Tableの行確定イベントで各項目の自動設定する。
   * 
   * @param e テーブルイベント
   * @throws Exception 例外発生時
   */
  @SuppressWarnings("unchecked")
  @Override
  public void rowValidating(TableEvent e) throws Exception {
    //イベント送出元が「tableEmp」オブジェクトの場合
    if (e.getSource()==tableEmp) {
      Row<DB.Employee> row = e.getRow();
      row.startEditing();
      if (row.entity.createDatetime == null) {
        row.entity.createDatetime = new java.sql.Timestamp(System.currentTimeMillis());
      }
      row.entity.updateDatetime = new java.sql.Timestamp(System.currentTimeMillis());
    }
  }
  
  /**
   * ページでチェックされた行を社員テーブルオブジェクトから削除します。
   * 
   * @throws Exception 例外発生時
   */
  private void deleteRows() throws Exception {
    //先頭行へカーソル移動
    tableEmp.first();
    //カーソルの有効範囲内でループ
    while(tableEmp.inBounds()) {
      //カーソル行が削除予約されている場合
      if (tableEmp.getRow().isDeletionDeserved()) {
        //カーソル行を削除
        tableEmp.deleteRow();
      } else {
        //次の行へカーソル移動
        tableEmp.next();
      }
    }
  }

  /**
   * 社員テーブルオブジェクトに新規行を挿入します。
   * 
   * @throws Exception 例外発生時
   */
  public void insertEmpList() throws Exception {
    // 先頭ページへ移動
    tableEmp.pageFirst();
    // 新規行を挿入,選択中の所属コードをディフォルト設定
    tableEmp.insertRow(0)
      .entity.deptCode = tableEmp.getParams().getInt("deptCode");
    // 再表示
    show();
  }
  
  /**
   * ページ送信内容をチェックします。
   * 
   * @throws Exception 例外発生時
   */
  public void confirmEmpList() throws Exception {
    //リクエスト内容のチェック＆入力チェックがOKの場合、チェック行の削除
    deleteRows();
    //WebSQLTableオブジェクトにエラーリストがない場合、メッセージ設定
    if (tableEmp.getErrors().size() == 0) messages = "問題ありません。";
    //再表示
    show();
  }
  
  /**
   * 入力チェックを行います。
   * 
   * @return boolean エラー有無
   * @throws Exception 例外発生時
   */
//  private boolean hasParamsError() throws Exception {
//    boolean hasError = false;
//    //「所属」チェック 
//    if (tableEmp.entity.deptCode == null) {
//      tableEmp.addErrorMessage(Caption.DEPT_CODE + "を選択してください。");
//      hasError = true;
//    }
//    return hasError;
//  }
  
  /**
   * 変更内容とエラーをリセットします。
   * 
   * @throws Exception 例外発生時
   */
  @DefaultCheckReqest(isEnabled=false)
  public void cancelUpdates() throws Exception {
    //メッセージ初期化
    messages = "";
    //変更前の状態に戻す
    tableEmp.cancelUpdates(false);
    //再表示
    show();  
  }
  
  /**
   * 社員テーブルオブジェクトからデータベースへ値を更新します。
   * 
   * @throws Exception 例外発生時
   */
  public void saveEmpList() throws Exception {
    // チェック行を削除
    deleteRows();
    // コネクション取得
    Connection conn = getConnection();
    try {
      for (Row<DB.Employee> row : tableEmp.visibleRows()) {
        // 新規登録の場合
        if (row.getStatus().equals(Row.Status.CREATING) || row.getStatus().equals(Row.Status.CREATED)) {
          // EMP_IDを採番する
          updateEmpSeq.execute(conn, true, false);
          tableEmpSeq.activate(conn);
          // 採番したEMP_IDを設定
          row.startEditing();
          row.entity.id = tableEmpSeq.entity.value;
          row.validate();
        }
      }
      // 社員テーブルの変更内容をデータベースに登録する
      tableEmp.resolveChanges(conn);
      // トランザクションをコミット
      conn.commit();
      // メッセージ設定
      messages = "登録しました。";
    } catch (Exception x) {
      // トランザクションをロールバック
      conn.rollback();
      // 例外時処理
      x.printStackTrace();
      System.out.println(x.getLocalizedMessage());
      tableEmp.addErrorMessage(x.getLocalizedMessage());
      throw new LSApplicationException(x);
    } finally {
      // コネクションを閉じる
      conn.close();
      // 再表示
      show();
    }
  }

  /**
   * 一覧情報をCSVに出力します。
   * 
   * @throws Exception 例外発生時
   */
  public void csvEmpList() throws Exception {
    String escapeStr = new String();
    getResponse().setContentType("application/octet-stream;charset=" + "Windows-31J");
    getResponse().setHeader("Content-Disposition", "attachment; filename=emp.csv");
    try {
      PrintWriter out = getResponse().getWriter();
      // カラムのカウントを取得
      int cnt = tableEmp.getColumnCount();
      // キャプションの編集
      StringBuffer strCap = new StringBuffer();
      for (int i = 1; i < cnt; i++) {
        if (i != 1) {
          strCap.append(",");
        }
        strCap.append("\"" + tableEmp.getColumn(i).getCaption() + "\"");
      }
      out.println(strCap.toString());
      // 値の編集
      for (Row<DB.Employee> row : tableEmp.visibleRows()) {
        StringBuffer strVal = new StringBuffer();
        for (int i = 1; i < cnt; i++) {
          if (i != 1) {
            strVal.append(",");
          }
          if (tableEmp.getColumn(i).getType() == Column.Type.STRING) {
            escapeStr = row.getValueAsString(i).replaceAll(new String("\r\n"), "\n");
            escapeStr = escapeStr.replaceAll("\"", "\"\"");
            strVal.append("\"" + escapeStr + "\"");
          } else {
            escapeStr = row.getValueAsString(i).replaceAll(",", ""); // 数値内のカンマ(,)を削除
            strVal.append(escapeStr);
          }
        }
        out.println(strVal.toString());
      }
    } finally {
    }
  }
}
